@@ -27,7 +27,7 @@ typedef struct {
 	GMainContext *async_context;
 	gboolean      use_thread_context;
 
-	SoupMessageQueueItem *cur_item;
+	SoupMessage *current_msg;
 	SoupConnectionState state;
 	time_t       unused_timeout;
 	guint        io_timeout, idle_timeout;
@@ -60,7 +60,6 @@ enum {
 	PROP_TIMEOUT,
 	PROP_IDLE_TIMEOUT,
 	PROP_STATE,
-	PROP_MESSAGE,
 
 	LAST_PROP
 };
@@ -199,12 +198,6 @@ soup_connection_get_property (GObject *object, guint prop_id,
 	case PROP_STATE:
 		g_value_set_enum (value, priv->state);
 		break;
-	case PROP_MESSAGE:
-		if (priv->cur_item)
-			g_value_set_object (value, priv->cur_item->msg);
-		else
-			g_value_set_object (value, NULL);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -321,13 +314,6 @@ soup_connection_class_init (SoupConnectionClass *connection_class)
 				   "Current state of connection",
 				   SOUP_TYPE_CONNECTION_STATE, SOUP_CONNECTION_NEW,
 				   G_PARAM_READWRITE));
-	g_object_class_install_property (
-		object_class, PROP_MESSAGE,
-		g_param_spec_object (SOUP_CONNECTION_MESSAGE,
-				     "Message",
-				     "Message being processed",
-				     SOUP_TYPE_MESSAGE,
-				     G_PARAM_READABLE));
 }
 
 static void
@@ -374,37 +360,62 @@ stop_idle_timer (SoupConnectionPrivate *priv)
 }
 
 static void
-current_item_restarted (SoupMessage *msg, gpointer user_data)
+current_msg_got_body (SoupMessage *msg, gpointer user_data)
 {
 	SoupConnection *conn = user_data;
 	SoupConnectionPrivate *priv = SOUP_CONNECTION_GET_PRIVATE (conn);
 
 	priv->unused_timeout = 0;
+
+	if (priv->proxy_uri &&
+	    msg->method == SOUP_METHOD_CONNECT &&
+	    SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
+		soup_connection_event (conn, G_SOCKET_CLIENT_PROXY_NEGOTIATED, NULL);
+
+		/* We're now effectively no longer proxying */
+		g_clear_pointer (&priv->proxy_uri, soup_uri_free);
+	}
+
+	priv->reusable = soup_message_is_keepalive (msg);
 }
 
 static void
-set_current_item (SoupConnection *conn, SoupMessageQueueItem *item)
+clear_current_msg (SoupConnection *conn)
+{
+	SoupConnectionPrivate *priv = SOUP_CONNECTION_GET_PRIVATE (conn);
+	SoupMessage *msg;
+
+	msg = priv->current_msg;
+	priv->current_msg = NULL;
+
+	g_signal_handlers_disconnect_by_func (msg, G_CALLBACK (current_msg_got_body), conn);
+	g_object_unref (msg);
+}
+
+static void
+set_current_msg (SoupConnection *conn, SoupMessage *msg)
 {
 	SoupConnectionPrivate *priv = SOUP_CONNECTION_GET_PRIVATE (conn);
 
-	g_return_if_fail (priv->cur_item == NULL);
+	g_return_if_fail (priv->state == SOUP_CONNECTION_IN_USE);
 
 	g_object_freeze_notify (G_OBJECT (conn));
 
+	if (priv->current_msg) {
+		g_return_if_fail (priv->current_msg->method == SOUP_METHOD_CONNECT);
+		clear_current_msg (conn);
+	}
+
 	stop_idle_timer (priv);
 
-	item->state = SOUP_MESSAGE_RUNNING;
-	priv->cur_item = item;
-	g_object_notify (G_OBJECT (conn), "message");
+	priv->current_msg = g_object_ref (msg);
 	priv->reusable = FALSE;
 
-	g_signal_connect (item->msg, "restarted",
-			  G_CALLBACK (current_item_restarted), conn);
+	g_signal_connect (msg, "got-body",
+			  G_CALLBACK (current_msg_got_body), conn);
 
-	if (item->msg->method == SOUP_METHOD_CONNECT)
+	if (priv->proxy_uri && msg->method == SOUP_METHOD_CONNECT)
 		soup_connection_event (conn, G_SOCKET_CLIENT_PROXY_NEGOTIATING, NULL);
-	else if (priv->state == SOUP_CONNECTION_IDLE)
-		soup_connection_set_state (conn, SOUP_CONNECTION_IN_USE);
 
 	g_object_thaw_notify (G_OBJECT (conn));
 }
@@ -422,12 +433,6 @@ proxy_socket_event (SoupSocket          *socket,
 		soup_connection_event (conn, event, connection);
 }
 
-static void
-socket_disconnected (SoupSocket *sock, gpointer conn)
-{
-	soup_connection_disconnect (conn);
-}
-
 typedef struct {
 	SoupConnection *conn;
 	SoupConnectionCallback callback;
@@ -445,9 +450,6 @@ socket_connect_finished (SoupSocket *socket, guint status, gpointer user_data)
 	g_signal_handler_disconnect (socket, data->event_id);
 
 	if (SOUP_STATUS_IS_SUCCESSFUL (status)) {
-		g_signal_connect (priv->socket, "disconnected",
-				  G_CALLBACK (socket_disconnected), data->conn);
-
 		if (priv->ssl && !priv->proxy_uri) {
 			soup_connection_event (data->conn,
 					       G_SOCKET_CLIENT_TLS_HANDSHAKED,
@@ -472,6 +474,8 @@ socket_connect_finished (SoupSocket *socket, guint status, gpointer user_data)
 			status = soup_status_proxify (status);
 		data->callback (data->conn, status, data->callback_data);
 	}
+	if (!SOUP_STATUS_IS_SUCCESSFUL (status) && status != SOUP_STATUS_TRY_AGAIN)
+		soup_connection_disconnect (data->conn);
 	g_object_unref (data->conn);
 	if (data->cancellable)
 		g_object_unref (data->cancellable);
@@ -637,7 +641,6 @@ soup_connection_connect_sync (SoupConnection *conn, GCancellable *cancellable)
 				 SOUP_SOCKET_CLEAN_DISPOSE, TRUE,
 				 NULL);
 	g_object_unref (remote_addr);
-
 	event_id = g_signal_connect (priv->socket, "event",
 				     G_CALLBACK (proxy_socket_event), conn);
 	status = soup_socket_connect_sync (priv->socket, cancellable);
@@ -668,9 +671,6 @@ soup_connection_connect_sync (SoupConnection *conn, GCancellable *cancellable)
 	}
 
 	if (SOUP_STATUS_IS_SUCCESSFUL (status)) {
-		g_signal_connect (priv->socket, "disconnected",
-				  G_CALLBACK (socket_disconnected), conn);
-
 		if (!priv->ssl || !priv->proxy_uri) {
 			soup_connection_event (conn,
 					       G_SOCKET_CLIENT_COMPLETE,
@@ -679,13 +679,15 @@ soup_connection_connect_sync (SoupConnection *conn, GCancellable *cancellable)
 		soup_connection_set_state (conn, SOUP_CONNECTION_IN_USE);
 		priv->unused_timeout = time (NULL) + SOUP_CONNECTION_UNUSED_TIMEOUT;
 		start_idle_timer (conn);
-	} else {
+	} else if (status != SOUP_STATUS_TRY_AGAIN) {
 	fail:
 		if (priv->socket) {
 			soup_socket_disconnect (priv->socket);
 			g_object_unref (priv->socket);
 			priv->socket = NULL;
 		}
+
+		soup_connection_disconnect (conn);
 	}
 
 	if (priv->socket)
@@ -719,8 +721,10 @@ soup_connection_start_ssl_sync (SoupConnection *conn,
 
 	if (!soup_socket_start_proxy_ssl (priv->socket,
 					  priv->remote_uri->host,
-					  cancellable))
+					  cancellable)) {
+		soup_connection_disconnect (conn);
 		return SOUP_STATUS_SSL_FAILED;
+	}
 
 	soup_connection_event (conn, G_SOCKET_CLIENT_TLS_HANDSHAKING, NULL);
 	status = soup_socket_handshake_sync (priv->socket, cancellable);
@@ -732,6 +736,9 @@ soup_connection_start_ssl_sync (SoupConnection *conn,
 		status = SOUP_STATUS_TRY_AGAIN;
 	}
 
+	if (!SOUP_STATUS_IS_SUCCESSFUL (status) && status != SOUP_STATUS_TRY_AGAIN)
+		soup_connection_disconnect (conn);
+
 	return status;
 }
 
@@ -750,6 +757,8 @@ start_ssl_completed (SoupSocket *socket, guint status, gpointer user_data)
 	}
 
 	data->callback (data->conn, status, data->callback_data);
+	if (!SOUP_STATUS_IS_SUCCESSFUL (status) && status != SOUP_STATUS_TRY_AGAIN)
+		soup_connection_disconnect (data->conn);
 	g_object_unref (data->conn);
 	g_slice_free (SoupConnectionAsyncConnectData, data);
 }
@@ -820,14 +829,9 @@ soup_connection_disconnect (SoupConnection *conn)
 		soup_connection_set_state (conn, SOUP_CONNECTION_DISCONNECTED);
 
 	if (priv->socket) {
-		/* Set the socket to NULL at the beginning to avoid reentrancy
-		 * issues. soup_socket_disconnect() could trigger a reentrant
-		 * call unref'ing and disconnecting the socket twice.
-		 */
 		SoupSocket *socket = priv->socket;
+
 		priv->socket = NULL;
-		g_signal_handlers_disconnect_by_func (socket,
-						      socket_disconnected, conn);
 		soup_socket_disconnect (socket);
 		g_object_unref (socket);
 	}
@@ -892,7 +896,6 @@ void
 soup_connection_set_state (SoupConnection *conn, SoupConnectionState state)
 {
 	SoupConnectionPrivate *priv;
-	SoupConnectionState old_state;
 
 	g_return_if_fail (SOUP_IS_CONNECTION (conn));
 	g_return_if_fail (state >= SOUP_CONNECTION_NEW &&
@@ -901,44 +904,20 @@ soup_connection_set_state (SoupConnection *conn, SoupConnectionState state)
 	g_object_freeze_notify (G_OBJECT (conn));
 
 	priv = SOUP_CONNECTION_GET_PRIVATE (conn);
-	old_state = priv->state;
-
-	if (old_state == SOUP_CONNECTION_IN_USE)
-		priv->unused_timeout = 0;
-
-	if (priv->cur_item) {
-		SoupMessageQueueItem *item;
 
+	if (priv->current_msg) {
 		g_warn_if_fail (state == SOUP_CONNECTION_IDLE ||
 				state == SOUP_CONNECTION_DISCONNECTED);
-
-		item = priv->cur_item;
-		priv->cur_item = NULL;
-		g_object_notify (G_OBJECT (conn), "message");
-
-		g_signal_handlers_disconnect_by_func (item->msg, G_CALLBACK (current_item_restarted), conn);
-
-		if (item->msg->method == SOUP_METHOD_CONNECT &&
-		    SOUP_STATUS_IS_SUCCESSFUL (item->msg->status_code)) {
-			soup_connection_event (conn, G_SOCKET_CLIENT_PROXY_NEGOTIATED, NULL);
-
-			/* We're now effectively no longer proxying */
-			soup_uri_free (priv->proxy_uri);
-			priv->proxy_uri = NULL;
-
-			/* Nor are we actually IDLE... */
-			if (state == SOUP_CONNECTION_IDLE)
-				state = SOUP_CONNECTION_IN_USE;
-		}
-
-		if (!soup_message_is_keepalive (item->msg) || !priv->reusable)
-			soup_connection_disconnect (conn);
+		clear_current_msg (conn);
 	}
 
-	if (priv->state == old_state && priv->state != state) {
+	if (state == SOUP_CONNECTION_IDLE && !priv->reusable) {
+		/* This will recursively call set_state() */
+		soup_connection_disconnect (conn);
+	} else {
 		priv->state = state;
 
-		if (state == SOUP_CONNECTION_IDLE)
+		if (priv->state == SOUP_CONNECTION_IDLE)
 			start_idle_timer (conn);
 
 		g_object_notify (G_OBJECT (conn), "state");
@@ -947,12 +926,6 @@ soup_connection_set_state (SoupConnection *conn, SoupConnectionState state)
 	g_object_thaw_notify (G_OBJECT (conn));
 }
 
-void
-soup_connection_set_reusable (SoupConnection *conn)
-{
-	SOUP_CONNECTION_GET_PRIVATE (conn)->reusable = TRUE;
-}
-
 gboolean
 soup_connection_get_ever_used (SoupConnection *conn)
 {
@@ -978,9 +951,10 @@ soup_connection_send_request (SoupConnection          *conn,
 	g_return_if_fail (SOUP_IS_CONNECTION (conn));
 	g_return_if_fail (item != NULL);
 	priv = SOUP_CONNECTION_GET_PRIVATE (conn);
-	g_return_if_fail (priv->state != SOUP_CONNECTION_NEW && priv->state != SOUP_CONNECTION_DISCONNECTED);
+	g_return_if_fail (priv->state != SOUP_CONNECTION_NEW &&
+			  priv->state != SOUP_CONNECTION_DISCONNECTED);
 
-	if (item != priv->cur_item)
-		set_current_item (conn, item);
+	if (item->msg != priv->current_msg)
+		set_current_msg (conn, item->msg);
 	soup_message_send_request (item, completion_cb, user_data);
 }
