@@ -11,19 +11,20 @@
 G_BEGIN_DECLS
 
 SOUP_AVAILABLE_IN_2_26
-SOUP_DEPRECATED_IN_2_42_FOR(SoupSession:proxy-resolver)
 GType soup_proxy_resolver_gnome_get_type (void);
 #define SOUP_TYPE_PROXY_RESOLVER_GNOME (soup_proxy_resolver_gnome_get_type ())
 
 SOUP_AVAILABLE_IN_2_26
-SOUP_DEPRECATED_IN_2_42
 GType soup_gnome_features_2_26_get_type (void);
 #define SOUP_TYPE_GNOME_FEATURES_2_26 (soup_gnome_features_2_26_get_type ())
 
+#ifndef G_OS_WIN32
+#ifdef LIBSOUP_I_HAVE_READ_BUG_594377_AND_KNOW_SOUP_PASSWORD_MANAGER_MIGHT_GO_AWAY
 SOUP_AVAILABLE_IN_2_28
-SOUP_DEPRECATED_IN_2_28
 GType soup_password_manager_gnome_get_type (void);
 #define SOUP_TYPE_PASSWORD_MANAGER_GNOME (soup_password_manager_gnome_get_type ())
+#endif /* LIBSOUP_I_HAVE_READ_BUG_594377_AND_KNOW_SOUP_PASSWORD_MANAGER_MIGHT_GO_AWAY */
+#endif
 
 G_END_DECLS
 
