@@ -33,15 +33,16 @@ void                soup_message_headers_remove   (SoupMessageHeaders *hdrs,
 						   const char         *name);
 void                soup_message_headers_clear    (SoupMessageHeaders *hdrs);
 
+SOUP_AVAILABLE_IN_2_36
 void                soup_message_headers_clean_connection_headers (SoupMessageHeaders *hdrs);
 
-#ifndef LIBSOUP_DISABLE_DEPRECATED
-G_DEPRECATED_FOR(soup_message_headers_get_one or soup_message_headers_get_list)
+SOUP_DEPRECATED_IN_2_28_FOR ("soup_message_headers_get_one or soup_message_headers_get_list")
 const char         *soup_message_headers_get      (SoupMessageHeaders *hdrs,
 						   const char         *name);
-#endif
+SOUP_AVAILABLE_IN_2_28
 const char         *soup_message_headers_get_one  (SoupMessageHeaders *hdrs,
 						   const char         *name);
+SOUP_AVAILABLE_IN_2_28
 const char         *soup_message_headers_get_list (SoupMessageHeaders *hdrs,
 						   const char         *name);
 
@@ -97,38 +98,48 @@ typedef struct {
 	goffset end;
 } SoupRange;
 
+SOUP_AVAILABLE_IN_2_26
 gboolean        soup_message_headers_get_ranges          (SoupMessageHeaders  *hdrs,
 							  goffset              total_length,
 							  SoupRange          **ranges,
 							  int                 *length);
+SOUP_AVAILABLE_IN_2_26
 void            soup_message_headers_free_ranges         (SoupMessageHeaders  *hdrs,
 							  SoupRange           *ranges);
+SOUP_AVAILABLE_IN_2_26
 void            soup_message_headers_set_ranges          (SoupMessageHeaders  *hdrs,
 							  SoupRange           *ranges,
 							  int                  length);
+SOUP_AVAILABLE_IN_2_26
 void            soup_message_headers_set_range           (SoupMessageHeaders  *hdrs,
 							  goffset              start,
 							  goffset              end);
 
+SOUP_AVAILABLE_IN_2_26
 gboolean        soup_message_headers_get_content_range   (SoupMessageHeaders  *hdrs,
 							  goffset             *start,
 							  goffset             *end,
 							  goffset             *total_length);
+SOUP_AVAILABLE_IN_2_26
 void            soup_message_headers_set_content_range   (SoupMessageHeaders  *hdrs,
 							  goffset              start,
 							  goffset              end,
 							  goffset              total_length);
 
 
+SOUP_AVAILABLE_IN_2_26
 const char *soup_message_headers_get_content_type     (SoupMessageHeaders  *hdrs,
 						       GHashTable         **params);
+SOUP_AVAILABLE_IN_2_26
 void        soup_message_headers_set_content_type     (SoupMessageHeaders  *hdrs,
 						       const char          *content_type,
 						       GHashTable          *params);
 
+SOUP_AVAILABLE_IN_2_26
 gboolean soup_message_headers_get_content_disposition (SoupMessageHeaders  *hdrs,
 						       char               **disposition,
 						       GHashTable         **params);
+SOUP_AVAILABLE_IN_2_26
 void     soup_message_headers_set_content_disposition (SoupMessageHeaders  *hdrs,
 						       const char          *disposition,
 						       GHashTable          *params);
