@@ -8,7 +8,9 @@
 
 #include "soup-message.h"
 #include "soup-auth.h"
+#include "soup-content-processor.h"
 #include "soup-content-sniffer.h"
+#include "soup-session.h"
 
 typedef struct {
 	gpointer           io_data;
@@ -31,7 +33,6 @@ typedef struct {
 	SoupAuth          *auth, *proxy_auth;
 
 	GSList            *disabled_features;
-	GSList            *decoders;
 
 	SoupURI           *first_party;
 
@@ -65,13 +66,16 @@ void soup_message_read_request (SoupMessage               *req,
 				gpointer                   user_data);
 
 void soup_message_io_client    (SoupMessageQueueItem      *item,
+				GIOStream                 *iostream,
+				GMainContext              *async_context,
 				SoupMessageGetHeadersFn    get_headers_cb,
 				SoupMessageParseHeadersFn  parse_headers_cb,
 				gpointer                   headers_data,
 				SoupMessageCompletionFn    completion_cb,
 				gpointer                   user_data);
 void soup_message_io_server    (SoupMessage               *msg,
-				SoupSocket                *sock,
+				GIOStream                 *iostream,
+				GMainContext              *async_context,
 				SoupMessageGetHeadersFn    get_headers_cb,
 				SoupMessageParseHeadersFn  parse_headers_cb,
 				gpointer                   headers_data,
@@ -94,6 +98,25 @@ void                soup_message_io_pause       (SoupMessage          *msg);
 void                soup_message_io_unpause     (SoupMessage          *msg);
 gboolean            soup_message_io_in_progress (SoupMessage          *msg);
 
+gboolean soup_message_io_run_until_write  (SoupMessage   *msg,
+					   GCancellable  *cancellable,
+					   GError       **error);
+gboolean soup_message_io_run_until_read   (SoupMessage   *msg,
+					   GCancellable  *cancellable,
+					   GError       **error);
+gboolean soup_message_io_run_until_finish (SoupMessage   *msg,
+					   GCancellable  *cancellable,
+					   GError       **error);
+
+typedef gboolean (*SoupMessageSourceFunc) (SoupMessage *, gpointer);
+GSource *soup_message_io_get_source       (SoupMessage           *msg,
+					   GCancellable          *cancellable,
+					   SoupMessageSourceFunc  callback,
+					   gpointer               user_data);
+
+GInputStream *soup_message_io_get_response_istream (SoupMessage  *msg,
+						    GError      **error);
+
 gboolean soup_message_disables_feature (SoupMessage *msg,
 					gpointer     feature);
 
@@ -104,4 +127,9 @@ void soup_message_network_event (SoupMessage         *msg,
 				 GSocketClientEvent   event,
 				 GIOStream           *connection);
 
+GInputStream *soup_message_setup_body_istream (GInputStream *body_stream,
+					       SoupMessage *msg,
+					       SoupSession *session,
+					       SoupProcessingStage start_at_stage);
+
 #endif /* SOUP_MESSAGE_PRIVATE_H */
