@@ -162,8 +162,6 @@ soup_message_finalize (GObject *object)
 
 	g_slist_free (priv->disabled_features);
 
-	g_slist_free_full (priv->decoders, g_object_unref);
-
 	g_clear_object (&priv->tls_certificate);
 
 	soup_message_body_free (msg->request_body);
@@ -391,7 +389,7 @@ soup_message_class_init (SoupMessageClass *message_class)
 	 * every successful write() call, not only after finishing a
 	 * complete "chunk".
 	 *
-	 * Since: 2.4.1
+	 * Since: 2.24
 	 **/
 	signals[WROTE_BODY_DATA] =
 		g_signal_new ("wrote_body_data",
@@ -556,7 +554,7 @@ soup_message_class_init (SoupMessageClass *message_class)
 	 * that sniffing could be done is delivered on the first
 	 * emission of #SoupMessage::got-chunk.
 	 *
-	 * Since: 2.27.3
+	 * Since: 2.28
 	 **/
 	signals[CONTENT_SNIFFED] =
 		g_signal_new ("content_sniffed",
@@ -1163,6 +1161,11 @@ header_handler_metamarshal (GClosure *closure, GValue *return_value,
 	const char *header_name = marshal_data;
 	SoupMessageHeaders *hdrs;
 
+#ifdef FIXME
+	if (priv->io_status != SOUP_MESSAGE_IO_STATUS_RUNNING)
+		return;
+#endif
+
 	hdrs = priv->server_side ? msg->request_headers : msg->response_headers;
 	if (soup_message_headers_get_one (hdrs, header_name)) {
 		closure->marshal (closure, return_value, n_param_values,
@@ -1180,11 +1183,16 @@ header_handler_metamarshal (GClosure *closure, GValue *return_value,
  * @user_data: data to pass to @handler_cb
  *
  * Adds a signal handler to @msg for @signal, as with
- * g_signal_connect(), but the @callback will only be run if @msg's
- * incoming messages headers (that is, the
- * <literal>request_headers</literal> for a client #SoupMessage, or
- * the <literal>response_headers</literal> for a server #SoupMessage)
- * contain a header named @header.
+ * g_signal_connect(), but with two differences: the @callback will
+ * only be run if @msg has a header named @header, and it will only be
+ * run if no earlier handler cancelled or requeued the message.
+ *
+ * If @signal is one of the "got" signals (eg, "got_headers"), or
+ * "finished" or "restarted", then @header is matched against the
+ * incoming message headers (that is, the #request_headers for a
+ * client #SoupMessage, or the #response_headers for a server
+ * #SoupMessage). If @signal is one of the "wrote" signals, then
+ * @header is matched against the outgoing message headers.
  *
  * Return value: the handler ID from g_signal_connect()
  **/
@@ -1222,6 +1230,11 @@ status_handler_metamarshal (GClosure *closure, GValue *return_value,
 	SoupMessage *msg = g_value_get_object (&param_values[0]);
 	guint status = GPOINTER_TO_UINT (marshal_data);
 
+#ifdef FIXME
+	if (priv->io_status != SOUP_MESSAGE_IO_STATUS_RUNNING)
+		return;
+#endif
+
 	if (msg->status_code == status) {
 		closure->marshal (closure, return_value, n_param_values,
 				  param_values, invocation_hint,
@@ -1238,8 +1251,9 @@ status_handler_metamarshal (GClosure *closure, GValue *return_value,
  * @user_data: data to pass to @handler_cb
  *
  * Adds a signal handler to @msg for @signal, as with
- * g_signal_connect(), but the @callback will only be run if @msg has
- * the status @status_code.
+ * g_signal_connect() but with two differences: the @callback will
+ * only be run if @msg has the status @status_code, and it will only
+ * be run if no earlier handler cancelled or requeued the message.
  *
  * @signal must be a signal that will be emitted after @msg's status
  * is set. For a client #SoupMessage, this means it can't be a "wrote"
@@ -1397,8 +1411,6 @@ soup_message_cleanup_response (SoupMessage *req)
 						   SOUP_ENCODING_CONTENT_LENGTH);
 	}
 
-	g_slist_free_full (priv->decoders, g_object_unref);
-	priv->decoders = NULL;
 	priv->msg_flags &= ~SOUP_MESSAGE_CONTENT_DECODED;
 
 	req->status_code = SOUP_STATUS_NONE;
@@ -1923,13 +1935,11 @@ soup_message_set_https_status (SoupMessage *msg, SoupConnection *conn)
  * @certificate: (out) (transfer none): @msg's TLS certificate
  * @errors: (out): the verification status of @certificate
  *
- * If @msg is using https (or attempted to use https but got
- * %SOUP_STATUS_SSL_FAILED), this retrieves the #GTlsCertificate
- * associated with its connection, and the #GTlsCertificateFlags
- * showing what problems, if any, have been found with that
- * certificate.
+ * If @msg is using https, this retrieves the #GTlsCertificate
+ * associated with its connection, and the #GTlsCertificateFlags showing
+ * what problems, if any, have been found with that certificate.
  *
- * Return value: %TRUE if @msg used/attempted https, %FALSE if not
+ * Return value: %TRUE if @msg uses https, %FALSE if not
  *
  * Since: 2.34
  */
