@@ -137,12 +137,9 @@ tunnel_connect (SoupSession *session, SoupMessageQueueItem *related)
 	if (SOUP_STATUS_IS_SUCCESSFUL (status)) {
 		if (!soup_connection_start_ssl_sync (conn, related->cancellable))
 			status = SOUP_STATUS_SSL_FAILED;
+		soup_message_set_https_status (related->msg, conn);
 	}
 
-	soup_message_set_https_status (related->msg, conn);
-	if (!SOUP_STATUS_IS_SUCCESSFUL (status))
-		soup_connection_disconnect (conn);
-
 	g_object_unref (conn);
 	return status;
 }
@@ -167,34 +164,33 @@ try_again:
 		try_pruning = FALSE;
 	}
 
-	if (soup_connection_get_state (item->conn) != SOUP_CONNECTION_NEW) {
+	if (soup_connection_get_state (item->conn) == SOUP_CONNECTION_IDLE) {
 		item->state = SOUP_MESSAGE_READY;
 		return;
 	}
 
-	status = soup_connection_connect_sync (item->conn, item->cancellable);
-	if (status == SOUP_STATUS_TRY_AGAIN) {
-		soup_connection_disconnect (item->conn);
-		soup_message_queue_item_set_connection (item, NULL);
-		goto try_again;
-	}
+	if (soup_connection_get_state (item->conn) == SOUP_CONNECTION_NEW) {
+		status = soup_connection_connect_sync (item->conn, item->cancellable);
+		if (status == SOUP_STATUS_TRY_AGAIN) {
+			soup_session_set_item_connection (session, item, NULL);
+			goto try_again;
+		}
 
-	soup_message_set_https_status (msg, item->conn);
+		soup_message_set_https_status (msg, item->conn);
 
-	if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
-		if (!msg->status_code)
-			soup_session_set_item_status (session, item, status);
-		item->state = SOUP_MESSAGE_FINISHING;
-		soup_connection_disconnect (item->conn);
-		soup_message_queue_item_set_connection (item, NULL);
-		return;
+		if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
+			if (!msg->status_code)
+				soup_session_set_item_status (session, item, status);
+			item->state = SOUP_MESSAGE_FINISHING;
+			soup_session_set_item_connection (session, item, NULL);
+			return;
+		}
 	}
 
 	if (soup_connection_is_tunnelled (item->conn)) {
 		status = tunnel_connect (session, item);
 		if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
-			soup_connection_disconnect (item->conn);
-			soup_message_queue_item_set_connection (item, NULL);
+			soup_session_set_item_connection (session, item, NULL);
 			if (status == SOUP_STATUS_TRY_AGAIN)
 				goto try_again;
 			soup_session_set_item_status (session, item, status);
@@ -273,7 +269,6 @@ process_queue_item (SoupMessageQueueItem *item)
 			item->state = SOUP_MESSAGE_FINISHED;
 			soup_message_finished (item->msg);
 			soup_session_unqueue_item (session, item);
-			g_cond_broadcast (&priv->cond);
 			break;
 
 		default:
