@@ -74,7 +74,7 @@ typedef struct {
 	SoupSession *session;
 } SoupSessionHost;
 static guint soup_host_uri_hash (gconstpointer key);
-gboolean soup_host_uri_equal (gconstpointer v1, gconstpointer v2);
+static gboolean soup_host_uri_equal (gconstpointer v1, gconstpointer v2);
 
 typedef struct {
 	GTlsDatabase *tlsdb;
@@ -96,9 +96,11 @@ typedef struct {
 	guint max_conns, max_conns_per_host;
 	guint io_timeout, idle_timeout;
 
-	/* Must hold the conn_lock before potentially creating a
-	 * new SoupSessionHost, or adding/removing a connection.
-	 * Must not emit signals or destroy objects while holding it.
+	/* Must hold the conn_lock before potentially creating a new
+	 * SoupSessionHost, adding/removing a connection,
+	 * disconnecting a connection, or moving a connection from
+	 * IDLE to IN_USE. Must not emit signals or destroy objects
+	 * while holding it.
 	 */
 	GMutex conn_lock;
 
@@ -108,10 +110,17 @@ typedef struct {
 	GResolver *resolver;
 
 	char **http_aliases, **https_aliases;
+
+	gboolean disposed;
 } SoupSessionPrivate;
 #define SOUP_SESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SOUP_TYPE_SESSION, SoupSessionPrivate))
 
 static void free_host (SoupSessionHost *host);
+static void connection_state_changed (GObject *object, GParamSpec *param,
+				      gpointer user_data);
+static void connection_disconnected (SoupConnection *conn, gpointer user_data);
+static void drop_connection (SoupSession *session, SoupSessionHost *host,
+			     SoupConnection *conn);
 
 static void auth_manager_authenticate (SoupAuthManager *manager,
 				       SoupMessage *msg, SoupAuth *auth,
@@ -217,7 +226,9 @@ soup_session_dispose (GObject *object)
 	SoupSession *session = SOUP_SESSION (object);
 	SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (session);
 
+	priv->disposed = TRUE;
 	soup_session_abort (session);
+	g_warn_if_fail (g_hash_table_size (priv->conns) == 0);
 
 	while (priv->features)
 		soup_session_remove_feature (session, priv->features->data);
@@ -465,7 +476,13 @@ soup_session_set_property (GObject *object, guint prop_id,
 		uri = g_value_get_boxed (value);
 
 		if (uri) {
+#ifdef G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+#endif
 			soup_session_remove_feature_by_type (session, SOUP_TYPE_PROXY_RESOLVER);
+#ifdef G_GNUC_END_IGNORE_DEPRECATIONS
+G_GNUC_END_IGNORE_DEPRECATIONS
+#endif
 			feature = SOUP_SESSION_FEATURE (soup_proxy_resolver_static_new (uri));
 			soup_session_add_feature (session, feature);
 			g_object_unref (feature);
@@ -727,6 +744,10 @@ soup_session_get_async_context (SoupSession *session)
 
 /* Hosts */
 
+/* Note that we can't use soup_uri_host_hash() and soup_uri_host_equal()
+ * because we want to ignore the protocol; http://example.com and
+ * webcal://example.com are the same host.
+ */
 static guint
 soup_host_uri_hash (gconstpointer key)
 {
@@ -737,7 +758,7 @@ soup_host_uri_hash (gconstpointer key)
 	return uri->port + soup_str_case_hash (uri->host);
 }
 
-gboolean
+static gboolean
 soup_host_uri_equal (gconstpointer v1, gconstpointer v2)
 {
 	const SoupURI *one = v1;
@@ -818,8 +839,7 @@ get_host_for_message (SoupSession *session, SoupMessage *msg)
 static void
 free_host (SoupSessionHost *host)
 {
-	g_slist_free_full (host->connections,
-			   (GDestroyNotify) soup_connection_disconnect);
+	g_warn_if_fail (host->connections == NULL);
 
 	if (host->keep_alive_src) {
 		g_source_destroy (host->keep_alive_src);
@@ -1010,6 +1030,22 @@ redirect_handler (SoupMessage *msg, gpointer user_data)
 	soup_session_redirect_message (session, msg);
 }
 
+static void
+message_restarted (SoupMessage *msg, gpointer user_data)
+{
+	SoupMessageQueueItem *item = user_data;
+
+	if (item->conn &&
+	    (!soup_message_is_keepalive (msg) ||
+	     SOUP_STATUS_IS_REDIRECTION (msg->status_code))) {
+		if (soup_connection_get_state (item->conn) == SOUP_CONNECTION_IN_USE)
+			soup_connection_set_state (item->conn, SOUP_CONNECTION_IDLE);
+		soup_session_set_item_connection (item->session, item, NULL);
+	}
+
+	soup_message_cleanup_response (msg);
+}
+
 SoupMessageQueueItem *
 soup_session_append_queue_item (SoupSession *session, SoupMessage *msg,
 				SoupSessionCallback callback, gpointer user_data)
@@ -1032,6 +1068,8 @@ soup_session_append_queue_item (SoupSession *session, SoupMessage *msg,
 			msg, "got_body", "Location",
 			G_CALLBACK (redirect_handler), item);
 	}
+	g_signal_connect (msg, "restarted",
+			  G_CALLBACK (message_restarted), item);
 
 	g_signal_emit (session, signals[REQUEST_QUEUED], 0, msg);
 
@@ -1093,8 +1131,11 @@ soup_session_cleanup_connections (SoupSession *session,
 	while (g_hash_table_iter_next (&iter, &conn, &host)) {
 		state = soup_connection_get_state (conn);
 		if (state == SOUP_CONNECTION_REMOTE_DISCONNECTED ||
-		    (prune_idle && state == SOUP_CONNECTION_IDLE))
+		    (prune_idle && state == SOUP_CONNECTION_IDLE)) {
 			conns = g_slist_prepend (conns, g_object_ref (conn));
+			g_hash_table_iter_remove (&iter);
+			drop_connection (session, host, conn);
+		}
 	}
 	g_mutex_unlock (&priv->conn_lock);
 
@@ -1118,6 +1159,15 @@ free_unused_host (gpointer user_data)
 	SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (host->session);
 
 	g_mutex_lock (&priv->conn_lock);
+
+	/* In a multithreaded session, a connection might have been
+	 * added while we were waiting for conn_lock.
+	 */
+	if (host->connections) {
+		g_mutex_unlock (&priv->conn_lock);
+		return FALSE;
+	}
+
 	/* This will free the host in addition to removing it from the
 	 * hash table
 	 */
@@ -1131,17 +1181,15 @@ free_unused_host (gpointer user_data)
 }
 
 static void
-connection_disconnected (SoupConnection *conn, gpointer user_data)
+drop_connection (SoupSession *session, SoupSessionHost *host, SoupConnection *conn)
 {
-	SoupSession *session = user_data;
 	SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (session);
-	SoupSessionHost *host;
 
-	g_mutex_lock (&priv->conn_lock);
+	/* Note: caller must hold conn_lock, and must remove @conn
+	 * from priv->conns itself.
+	 */
 
-	host = g_hash_table_lookup (priv->conns, conn);
 	if (host) {
-		g_hash_table_remove (priv->conns, conn);
 		host->connections = g_slist_remove (host->connections, conn);
 		host->num_conns--;
 
@@ -1163,10 +1211,27 @@ connection_disconnected (SoupConnection *conn, gpointer user_data)
 	}
 
 	g_signal_handlers_disconnect_by_func (conn, connection_disconnected, session);
+	g_signal_handlers_disconnect_by_func (conn, connection_state_changed, session);
 	priv->num_conns--;
 
-	g_mutex_unlock (&priv->conn_lock);
 	g_object_unref (conn);
+}
+
+static void
+connection_disconnected (SoupConnection *conn, gpointer user_data)
+{
+	SoupSession *session = user_data;
+	SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (session);
+	SoupSessionHost *host;
+
+	g_mutex_lock (&priv->conn_lock);
+
+	host = g_hash_table_lookup (priv->conns, conn);
+	if (host)
+		g_hash_table_remove (priv->conns, conn);
+	drop_connection (session, host, conn);
+
+	g_mutex_unlock (&priv->conn_lock);
 
 	SOUP_SESSION_GET_CLASS (session)->kick (session);
 }
@@ -1194,7 +1259,7 @@ soup_session_make_connect_message (SoupSession    *session,
 	soup_message_set_flags (msg, SOUP_MESSAGE_NO_REDIRECT);
 
 	item = soup_session_append_queue_item (session, msg, NULL, NULL);
-	soup_message_queue_item_set_connection (item, conn);
+	soup_session_set_item_connection (session, item, conn);
 	g_object_unref (msg);
 	item->state = SOUP_MESSAGE_RUNNING;
 
@@ -1214,6 +1279,9 @@ soup_session_get_connection (SoupSession *session,
 	int num_pending = 0;
 	gboolean need_new_connection;
 
+	if (priv->disposed)
+		return FALSE;
+
 	if (item->conn) {
 		g_return_val_if_fail (soup_connection_get_state (item->conn) != SOUP_CONNECTION_DISCONNECTED, FALSE);
 		return TRUE;
@@ -1231,7 +1299,7 @@ soup_session_get_connection (SoupSession *session,
 		if (!need_new_connection && soup_connection_get_state (conns->data) == SOUP_CONNECTION_IDLE) {
 			soup_connection_set_state (conns->data, SOUP_CONNECTION_IN_USE);
 			g_mutex_unlock (&priv->conn_lock);
-			soup_message_queue_item_set_connection (item, conns->data);
+			soup_session_set_item_connection (session, item, conns->data);
 			soup_message_set_https_status (item->msg, item->conn);
 			return TRUE;
 		} else if (soup_connection_get_state (conns->data) == SOUP_CONNECTION_CONNECTING)
@@ -1294,7 +1362,7 @@ soup_session_get_connection (SoupSession *session,
 	}
 
 	g_mutex_unlock (&priv->conn_lock);
-	soup_message_queue_item_set_connection (item, conn);
+	soup_session_set_item_connection (session, item, conn);
 	return TRUE;
 }
 
@@ -1314,9 +1382,10 @@ soup_session_unqueue_item (SoupSession          *session,
 	SoupSessionHost *host;
 
 	if (item->conn) {
-		if (soup_connection_get_state (item->conn) == SOUP_CONNECTION_IN_USE)
+		if (item->msg->method != SOUP_METHOD_CONNECT ||
+		    !SOUP_STATUS_IS_SUCCESSFUL (item->msg->status_code))
 			soup_connection_set_state (item->conn, SOUP_CONNECTION_IDLE);
-		soup_message_queue_item_set_connection (item, NULL);
+		soup_session_set_item_connection (session, item, NULL);
 	}
 
 	if (item->state != SOUP_MESSAGE_FINISHED) {
@@ -1341,6 +1410,36 @@ soup_session_unqueue_item (SoupSession          *session,
 	soup_message_queue_item_unref (item);
 }
 
+static void
+proxy_connection_event (SoupConnection      *conn,
+			GSocketClientEvent   event,
+			GIOStream           *connection,
+			gpointer             user_data)
+{
+	SoupMessageQueueItem *item = user_data;
+
+	soup_message_network_event (item->msg, event, connection);
+}
+
+void
+soup_session_set_item_connection (SoupSession          *session,
+				  SoupMessageQueueItem *item,
+				  SoupConnection       *conn)
+{
+	if (item->conn) {
+		g_signal_handlers_disconnect_by_func (item->conn, proxy_connection_event, item);
+		g_object_unref (item->conn);
+	}
+
+	item->conn = conn;
+
+	if (item->conn) {
+		g_object_ref (item->conn);
+		g_signal_connect (item->conn, "event",
+				  G_CALLBACK (proxy_connection_event), item);
+	}
+}
+
 void
 soup_session_set_item_status (SoupSession          *session,
 			      SoupMessageQueueItem *item,
@@ -1650,8 +1749,11 @@ soup_session_abort (SoupSession *session)
 	g_mutex_lock (&priv->conn_lock);
 	conns = NULL;
 	g_hash_table_iter_init (&iter, priv->conns);
-	while (g_hash_table_iter_next (&iter, &conn, &host))
+	while (g_hash_table_iter_next (&iter, &conn, &host)) {
 		conns = g_slist_prepend (conns, g_object_ref (conn));
+		g_hash_table_iter_remove (&iter);
+		drop_connection (session, host, conn);
+	}
 	g_mutex_unlock (&priv->conn_lock);
 
 	for (c = conns; c; c = c->next) {
@@ -1685,23 +1787,23 @@ prefetch_uri (SoupSession *session, SoupURI *uri,
 }
 
 /**
-* soup_session_prepare_for_uri:
-* @session: a #SoupSession
-* @uri: a #SoupURI which may be required
-*
-* Tells @session that @uri may be requested shortly, and so the
-* session can try to prepare (resolving the domain name, obtaining
-* proxy address, etc.) in order to work more quickly once the URI is
-* actually requested.
-*
-* This method acts asynchronously, in @session's
-* #SoupSession:async_context. If you are using #SoupSessionSync and do
-* not have a main loop running, then you can't use this method.
-*
-* Since: 2.30
-*
-* Deprecated: 2.38: use soup_session_prefetch_dns() instead
-**/
+ * soup_session_prepare_for_uri:
+ * @session: a #SoupSession
+ * @uri: a #SoupURI which may be required
+ *
+ * Tells @session that @uri may be requested shortly, and so the
+ * session can try to prepare (resolving the domain name, obtaining
+ * proxy address, etc.) in order to work more quickly once the URI is
+ * actually requested.
+ *
+ * This method acts asynchronously, in @session's
+ * #SoupSession:async_context. If you are using #SoupSessionSync and do
+ * not have a main loop running, then you can't use this method.
+ *
+ * Since: 2.30
+ *
+ * Deprecated: 2.38: use soup_session_prefetch_dns() instead
+ **/
 void
 soup_session_prepare_for_uri (SoupSession *session, SoupURI *uri)
 {
@@ -2055,7 +2157,7 @@ soup_session_class_init (SoupSessionClass *session_class)
 	 * #SoupMessage::finished (and all of the other #SoupMessage
 	 * signals) may be invoked multiple times for a given message.
 	 *
-	 * Since: 2.4.1
+	 * Since: 2.24
 	 **/
 	signals[REQUEST_QUEUED] =
 		g_signal_new ("request-queued",
@@ -2098,7 +2200,7 @@ soup_session_class_init (SoupSessionClass *session_class)
 	 * #SoupSession::request_queued for a detailed description of the
 	 * message lifecycle within a session.
 	 *
-	 * Since: 2.4.1
+	 * Since: 2.24
 	 **/
 	signals[REQUEST_UNQUEUED] =
 		g_signal_new ("request-unqueued",
@@ -2251,7 +2353,7 @@ soup_session_class_init (SoupSessionClass *session_class)
 	 * if you want to ensure that all future connections will have
 	 * this timeout value.
 	 *
-	 * Since: 2.4.1
+	 * Since: 2.24
 	 **/
 	/**
 	 * SOUP_SESSION_IDLE_TIMEOUT:
@@ -2259,7 +2361,7 @@ soup_session_class_init (SoupSessionClass *session_class)
 	 * Alias for the #SoupSession:idle-timeout property. (The idle
 	 * connection lifetime.)
 	 *
-	 * Since: 2.4.1
+	 * Since: 2.24
 	 **/
 	g_object_class_install_property (
 		object_class, PROP_IDLE_TIMEOUT,
@@ -2288,7 +2390,7 @@ soup_session_class_init (SoupSessionClass *session_class)
 				      "Use NTLM",
 				      "Whether or not to use NTLM authentication",
 				      FALSE,
-				      G_PARAM_READWRITE));
+				      G_PARAM_READWRITE | G_PARAM_DEPRECATED));
 	/**
 	 * SoupSession:ssl-ca-file:
 	 *
@@ -2312,7 +2414,7 @@ soup_session_class_init (SoupSessionClass *session_class)
 				     "SSL CA file",
 				     "File containing SSL CA certificates",
 				     NULL,
-				     G_PARAM_READWRITE));
+				     G_PARAM_READWRITE | G_PARAM_DEPRECATED));
 	/**
 	 * SOUP_SESSION_USE_SYSTEM_CA_FILE:
 	 *
