@@ -5,8 +5,6 @@
 
 #include "test-utils.h"
 
-#include "libsoup/soup-connection.h"
-
 SoupServer *server;
 SoupURI *base_uri;
 GMutex server_mutex;
@@ -541,8 +539,6 @@ do_max_conns_test (void)
 	soup_test_session_abort_unref (session);
 }
 
-GMainLoop *loop;
-
 static void
 np_request_started (SoupSession *session, SoupMessage *msg,
 		    SoupSocket *socket, gpointer user_data)
@@ -562,6 +558,13 @@ np_request_unqueued (SoupSession *session, SoupMessage *msg,
 		debug_printf (1, "    socket is still connected\n");
 		errors++;
 	}
+}
+
+static void
+np_request_finished (SoupSession *session, SoupMessage *msg,
+		     gpointer user_data)
+{
+	GMainLoop *loop = user_data;
 
 	g_main_loop_quit (loop);
 }
@@ -571,6 +574,7 @@ do_non_persistent_test_for_session (SoupSession *session)
 {
 	SoupMessage *msg;
 	SoupSocket *socket = NULL;
+	GMainLoop *loop;
 
 	loop = g_main_loop_new (NULL, FALSE);
 
@@ -584,7 +588,8 @@ do_non_persistent_test_for_session (SoupSession *session)
 	msg = soup_message_new_from_uri ("GET", base_uri);
 	soup_message_headers_append (msg->request_headers, "Connection", "close");
 	g_object_ref (msg);
-	soup_session_queue_message (session, msg, NULL, NULL);
+	soup_session_queue_message (session, msg,
+				    np_request_finished, loop);
 	g_main_loop_run (loop);
 	g_main_loop_unref (loop);
 
@@ -706,11 +711,10 @@ static void
 connection_state_changed (GObject *object, GParamSpec *param,
 			  gpointer user_data)
 {
-	SoupConnection *conn = SOUP_CONNECTION (object);
 	SoupConnectionState *state = user_data;
 	SoupConnectionState new_state;
 
-	new_state = soup_connection_get_state (conn);
+	g_object_get (object, "state", &new_state, NULL);
 	if (state_transitions[*state] != new_state) {
 		debug_printf (1, "      Unexpected transition: %s -> %s\n",
 			      state_names[*state], state_names[new_state]);
@@ -724,12 +728,12 @@ connection_state_changed (GObject *object, GParamSpec *param,
 }
 
 static void
-connection_created (SoupSession *session, SoupConnection *conn,
+connection_created (SoupSession *session, GObject *conn,
 		    gpointer user_data)
 {
 	SoupConnectionState *state = user_data;
 
-	*state = soup_connection_get_state (conn);
+	g_object_get (conn, "state", state, NULL);
 	if (*state != SOUP_CONNECTION_NEW) {
 		debug_printf (1, "      Unexpected initial state: %d\n",
 			      *state);
