@@ -11,6 +11,10 @@
 #include <stdio.h>
 #include <sys/stat.h>
 
+#ifdef G_OS_WIN32
+#include <getopt.h>
+#endif
+
 #ifdef HAVE_MMAP
 #include <sys/mman.h>
 #endif
@@ -255,7 +259,6 @@ main (int argc, char **argv)
 	int ssl_port = SOUP_ADDRESS_ANY_PORT;
 	const char *ssl_cert_file = NULL, *ssl_key_file = NULL;
 
-	g_type_init ();
 	signal (SIGINT, quit);
 
 	while ((opt = getopt (argc, argv, "p:k:c:s:")) != -1) {
